<?php
/**
 * Helper class with public methods to access the license API:
 * check()
 * activate()
 * deactivate()
 *
 * @author RavanH
 * @version 1.6
 */

class Status301_License_Manager {

	/*************
	* PROPERTIES *
	*************/

	/**
	 * Remote post response data on check_license and de/activate_license.
	 *
	 * $license_data->success : boolean
	 * $license_data->license :
	 * -- 'valid' -> Your license key was succesfully activated for this site.
	 * -- 'inactive' -> Your license is not active for this site.
	 * -- 'deactivated' -> Your license key was succesfully deactivated for this site.
	 * -- 'expired' -> Your license key has expired on %s. + formated expires date
	 * -- 'invalid' -> May return this on trying to (de)activate an expired/revoked license, check error property for more info
	 * -- 'invalid_item_id' -> When check is done for a key that does not match the product id
	 * -- 'failed' -> General validation error, may return on trying to (de)activate an empty or incorrect key
	 * $license_data->error :
	 * -- 'expired' -> Your license key has expired on %s. + formated expires date
	 * -- 'disabled' | 'revoked' -> Your license key has been disabled.
	 * -- 'invalid' | 'missing' -> This appears to be an invalid license key.
	 * -- 'invalid_item_id' -> This appears to be an invalid license key for %s. + package name
	 * -- 'site_inactive' -> Your license is not active for this URL.
	 * -- 'item_name_mismatch' -> This appears to be an invalid license key for %s. + package name
	 * -- 'no_activations_left' -> Your license key has reached its activation limit.
	 * -- 'error' -> An error occurred, please try again.
	 * $license_data->expires :
	 * -- 'lifetime'
	 * -- timestamp, make human readable with
	 *    date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
	 * $license_data->license_limit :
	 * -- integer, max number of sites for this license
	 * $license_data->activations_left :
	 * -- integer, remaining sites to activate for this license
	 * $license_data->payment_id :
	 * -- integer, can be used to create a link to license management page like :
	 *    trailingslashit( $store_url ) . 'account/?action=manage_licenses&payment_id=' . $license_data->payment_id
	 * $license_data->item_id : ( same as input $item_id )
	 * -- integer, can be used to create a link to license renewal checkout page like :
	 *    trailingslashit( $store_url ) . 'checkout/?edd_license_key=' . $key . '&download_id=' . $license_data->item_id
	 *
	 * Remote post response data on check_license and de/activate_license.
	 * $license_data->license : ''
	 * $license_data->new_version : version string, can be beta version
	 * $license_data->stable_version : version string
	 * $license_data->last_updated : date string
	 * $license_data->download_link : url string
	 * Other data: homepage, banners (high/low), icons (1x,2x), tested (wp version), url (changelog), added (first publication date).
	 *
	 * @property object|null
	 */
	public $license_data;

	/**
	 * License data.
	 */

	/**
	 * License status for this site, reduced from property $license_data->license
	 * Can be used to set coherent admin status messages.
	 *
	 * Possible values: 'valid', 'inactive', 'expired' or 'invalid'
	 *
	 * @property string $status
	 */
	public $status = '';

	/**
	 * Error, deduced from $license_data->license and $license_data->error responses
	 * Can be used to set coherent admin error message.
	 *
	 * Possible values: see $license_data->error
	 *
	 * @property string $error
	 */
	public $error = '';

	/**
	 * Expiration date, populated with the property $license_data->expires
	 * Note: ONLY set when the license is for current product ID,
	 *       NOT when the license is for another product (even thought such a response passes expiration data) !
	 *
	 * A timestamp or 'lifetime', make the timestamp human readable with
	 * date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
	 *
	 * @property string $expires
	 */
	public $expires = '';

	/**
	 * Payement ID, populated with the property $license_data->payment_id
	 * Note: Only set when the license is valid and for current product ID,
	 *       NOT when the license is for another product (even though such a response passes a payment_id) !
	 *
	 * Can be used to create link to account license management page:
	 * trailingslashit( $store_url ) . 'account/?action=manage_licenses&payment_id=' . $license_data->payment_id
	 *
	 * @property int $payment_id
	 */
	public $payment_id = 0;

	/**
	 * New version, populated with the property $license_data->new_version
	 * Note: Only set when the version is checked
	 *
	 * @property string $new_version
	 */
	public $new_version = '';

	/**
	 * Stable version, populated with the property $license_data->stable_version
	 * Note: Only set when the version is checked
	 *
	 * @property string $stable_version
	 */
	public $stable_version = '';

	/**
	 * Last updated date, populated with the property $license_data->last_updated
	 * Note: Only set when the version is checked
	 *
	 * @property string $last_updated
	 */
	public $last_updated = '';

	/**
	 * Download link, populated with the property $license_data->download_link
	 * Note: Only set when the version is checked
	 *
	 * @property string $download_link
	 */
	public $download_link = '';

	private $key = '';
	private $store_url = '';
	private $item_id = 0;
	private $item_name = '';
	private $beta = false;
	private $sslverify = false;
	private $timeout = 15;

	/**********
	* METHODS *
	**********/

	/**
	 * Class constructor.
	 *
	 * @uses trailingslashit()
	 *
	 * @param string  $_key        The license key.
	 * @param string  $_store_url  Download store URL.
	 * @param int     $_item_id    Download item ID.
	 * @param string  $_item_name  Download item name, only needed when no ID is provided.
	 * @param bool    $_beta       Wether to check for beta version.
	 */
	function __construct( $_key = '', $_store_url = '', $_item_id = 0, $_item_name = '', $_beta = false )
	{
		// populate
		$this->key = trim( $_key );
		$this->store_url = trailingslashit( $_store_url );
		$this->item_id = urlencode( (int) $_item_id );
		$this->item_name = urlencode( trim( $_item_name ) );
		$this->beta = (bool) $_beta;
	}

	/**
	 * Check the plugin version.
	 *
	 * Populates public property $license_data with the
	 * json response object (or null) if one is received and returns:
	 * -- true on valid new_version
	 * -- false on missing new_version (check $this->license_data->error for error code)
	 * -- response status code (other than 200) on communication failure
	 * -- WP_Error object if encountered.
	 *
	 * @since 1.6
	 *
	 * @uses $this->call(), is_object(), property_exists()
	 *
	 * @return bool|int|object
	 */
	public function version()
	{
		// Call the custom API with action get_version
		$response = $this->call( 'get_version' );

		if ( true === $response ) {
			return property_exists( $this->license_data, 'new_version' ) || property_exists( $this->license_data, 'stable_version' );
		}

		return $response;
	}

	/**
	 * Check the license key.
	 *
	 * Populates public property $license_data with the
	 * json response object (or null) if one is received and returns:
	 * -- true on valid license
	 * -- false on invalid license (check $this->license_data->error for error code)
	 * -- response status code (other than 200) on communication failure
	 * -- WP_Error object if encountered.
	 *
	 * @since 1.0
	 *
	 * @uses $this->call(), is_object(), property_exists()
	 *
	 * @return bool|int|object
	 */
	public function check()
	{
		// Call the custom API with action check_license
		$response = $this->call( 'check_license' );

		if ( true === $response ) {
			return property_exists( $this->license_data, 'success' ) && $this->license_data->success;
		}

		return $response;
	}

	/**
	 * Deactivate the license key.
	 *
	 * Populates public property $license_data with the
	 * json response object (or null) if one is received and returns:
	 * -- true on success
	 * -- false on failure (check $this->license_data->error for deactivation error)
	 * -- response status code (other than 200) on communication failure
	 * -- WP_Error object if encountered.
	 *
	 * @since 1.0
	 *
	 * @uses $this->call()
	 *
	 * @return bool|int|object
	 */
	public function deactivate()
	{
		// Call the custom API with action deactivate_license
		$response = $this->call( 'deactivate_license' );

		if ( true === $response ) {
			return property_exists( $this->license_data, 'success' ) && $this->license_data->success;
		}

		return $response;
	}

	/**
	 * Activate the license key.
	 *
	 * Populates public property $license_data with the
	 * json response object (or null) if one is received and returns:
	 * -- true on success
	 * -- false on failure (check $this->license_data->error for deactivation error)
	 * -- response status code (other than 200) on communication failure
	 * -- WP_Error object if encountered.
	 *
	 * @since 1.0
	 *
	 * @uses $this->call()
	 *
	 * @return bool|int|object
	 */
	public function activate()
	{
		// Call the custom API with action deactivate_license
		$response = $this->call( 'activate_license' );

		if ( true === $response ) {
			return is_object( $this->license_data )	&& property_exists( $this->license_data, 'success' ) && $this->license_data->success;
		}

		return $response;
	}

	/**
	 * Call home with an action.
	 *
	 * Accepts an action value, populating public property $license_data with the
	 * json response object (or null) if one is received and returns:
	 * -- true on success
	 * -- response status code (other than 200) on communication failure
	 * -- WP_Error object if encountered.
	 *
	 * @since 1.0
	 *
	 * @uses wp_remote_post(), wp_remote_retrieve_response_code(), json_decode(), wp_remote_retrieve_body()
	 *
	 * @param var $action The API action.
	 * @return bool|int|object true, response status code or WP_Error
	 */
	private function call( $_action )
	{

		if ( $this->store_url == trailingslashit ( home_url() ) ) {
			return 999; // Don't allow a plugin to ping itself
		}

		// data to send in our API request
		$api_params = array(
			'edd_action'  => $_action,
			'license'     => $this->key,
			'url'         => home_url(),
			'environment' => function_exists( 'wp_get_environment_type' ) ? wp_get_environment_type() : 'production'
		);

		if ( ! empty( $this->item_id ) ) $api_params['item_id'] = $this->item_id;
		elseif ( ! empty( $this->item_name ) ) $api_params['item_name'] = $this->item_name;
		else return 1000; // Don't allow a non-identifiable API call

		if ( ! empty( $this->beta ) ) $api_params['beta'] = '1';

		// Call the custom API.
		$response = wp_remote_post(
			$this->store_url,
			array(
				'timeout' => $this->timeout,
				'sslverify' => (bool) apply_filters( 'edd_sl_api_request_verify_ssl', $this->sslverify, $this ),
				'body' => $api_params
			)
		);

		if ( is_wp_error( $response ) ) {
			return $response;
		}

		$response_code = wp_remote_retrieve_response_code( $response );

		if ( 200 !== $response_code ) {
			return $response_code;
		}

		$this->license_data = json_decode( wp_remote_retrieve_body( $response ) );

		if ( ! is_object( $this->license_data ) ) {
			return false;
		}

		// process license data and populate properties we can find
		switch ( $this->license_data->license ) {
			case 'valid':
			case 'inactive':
				// license status
				$this->status = $this->license_data->license;
				// license payment id
				$this->payment_id = property_exists( $this->license_data, 'payment_id' ) && ! empty( $this->license_data->payment_id ) ? $this->license_data->payment_id : $this->payment_id;
				// license expiration timestamp or 'lifetime'
				$this->expires = property_exists( $this->license_data, 'expires' ) && ! empty( $this->license_data->expires ) ? $this->license_data->expires : $this->expires;
				break;

			case 'expired':
				// license status
				$this->status = $this->license_data->license;
				// license payment id
				$this->payment_id = property_exists( $this->license_data, 'payment_id' ) && ! empty( $this->license_data->payment_id ) ? $this->license_data->payment_id : $this->payment_id;
				// response error
				$this->error = property_exists( $this->license_data, 'error' ) && ! empty( $this->license_data->error ) ? $this->license_data->error : $this->license_data->license;
				// license expiration timestamp or 'lifetime'
				$this->expires = property_exists( $this->license_data, 'expires' ) && ! empty( $this->license_data->expires ) ? $this->license_data->expires : $this->expires;
				break;

			case 'deactivated':
			case 'site_inactive':
				// license status
				$this->status = 'inactive';
				// license payment id
				$this->payment_id = property_exists( $this->license_data, 'payment_id' ) && ! empty( $this->license_data->payment_id ) ? $this->license_data->payment_id : $this->payment_id;
				// license expiration timestamp or 'lifetime'
				$this->expires = property_exists( $this->license_data, 'expires' ) && ! empty( $this->license_data->expires ) ? $this->license_data->expires : $this->expires;
				break;

			case 'invalid':
			case 'invalid_item_id':
				// license status
				$this->status = 'invalid';
				// response error
				$this->error = property_exists( $this->license_data, 'error' ) && ! empty( $this->license_data->error ) ? $this->license_data->error : $this->license_data->license;
				// don't set expiration or payment id even if that data was passed by remote post response
				break;

			case 'failed':
				// Leave status unknown
				// response error or 'invalid'
				$this->error = property_exists( $this->license_data, 'error' ) && ! empty( $this->license_data->error ) ? $this->license_data->error : 'invalid';
				break;

			default:
				// Get version data.
				$this->new_version = property_exists( $this->license_data, 'new_version' ) && ! empty( $this->license_data->new_version ) ? $this->license_data->new_version : '';
				$this->stable_version = property_exists( $this->license_data, 'stable_version' ) && ! empty( $this->license_data->stable_version ) ? $this->license_data->stable_version : '';
				$this->last_updated = property_exists( $this->license_data, 'last_updated' ) && ! empty( $this->license_data->last_updated ) ? $this->license_data->last_updated : '';
				$this->download_link = property_exists( $this->license_data, 'download_link' ) && ! empty( $this->license_data->download_link ) ? $this->license_data->download_link : '';
				// Other possible data: homepage, banners (high/low), icons (1x,2x), tested (wp version), url (changelog), added (first publication date).

				$this->error = property_exists( $this->license_data, 'error' ) && ! empty( $this->license_data->error ) ? $this->license_data->error : '';
		}

		return true;
	}

}
